@@ -27,16 +27,17 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libgnomeui/gnome-icon-lookup.h>
-#include <libgnomeui/gnome-icon-theme.h>
 
 #include "typedefs.h"
 #include "gth-file-view.h"
 #include "gth-file-view-list.h"
 #include "gthumb-marshal.h"
 #include "gthumb-enum-types.h"
+#include "file-utils.h"
 #include "file-data.h"
 #include "icons/pixbufs.h"
 #include "pixbuf-utils.h"
+#include "gth-sort-utils.h"
 
 
 enum {
@@ -53,12 +54,15 @@ enum {
 
 struct _GthFileViewListPrivate {
 	GtkTreeView    *tree_view;
+	GtkTreeModel   *filter_model;
 	GtkListStore   *list_store;
 	GthSortMethod   sort_method;
 	GtkSortType     sort_type;
 	GthViewMode     view_mode;
+	GthVisibleFunc  filter_func;
+	gpointer        filter_data;
 	int             max_image_size;
-	GnomeIconTheme *icon_theme;
+	GtkIconTheme   *icon_theme;
 	gboolean        enable_thumbs;
 	gboolean        reorderable;
 	GdkPixbuf      *unknown_pixbuf_small;
@@ -140,7 +144,7 @@ get_sized_pixbuf (GthFileViewList *gfv_list,
 	if (gfv_list->priv->max_image_size == 0)
 		return NULL;
 
-	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 
+	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
 				 gfv_list->priv->max_image_size,
 				 gfv_list->priv->max_image_size);
 	gdk_pixbuf_fill (result, 0x00000000);
@@ -150,8 +154,8 @@ get_sized_pixbuf (GthFileViewList *gfv_list,
 
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
-	x = (gfv_list->priv->max_image_size - w) / 2; 
-	y = (gfv_list->priv->max_image_size - h) / 2; 
+	x = (gfv_list->priv->max_image_size - w) / 2;
+	y = (gfv_list->priv->max_image_size - h) / 2;
 
 	gdk_pixbuf_copy_area (pixbuf,
 			      0, 0,
@@ -165,7 +169,7 @@ get_sized_pixbuf (GthFileViewList *gfv_list,
 
 static void
 gfv_insert (GthFileView  *file_view,
-	    int           pos, 
+	    int           pos,
 	    GdkPixbuf    *pixbuf,
 	    const char   *text,
 	    const char   *comment)
@@ -273,9 +277,11 @@ gfv_append_with_data (GthFileView  *file_view,
 
 
 static void
-gfv_remove (GthFileView  *file_view, 
-	    int           pos)
+gfv_remove (GthFileView *file_view,
+	    gpointer     data)
 {
+	/* FIXME */
+	/*
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
 	GtkTreePath     *path;
@@ -290,6 +296,7 @@ gfv_remove (GthFileView  *file_view,
 	gtk_tree_path_free (path);
 
 	gtk_list_store_remove (gfv_list->priv->list_store, &iter);
+	*/
 }
 
 
@@ -305,6 +312,33 @@ gfv_clear (GthFileView  *file_view)
 }
 
 
+static gboolean
+get_list_store_iter_from_pos (GthFileViewList *gfv_list,
+			      int              pos,
+			      GtkTreeIter     *iter)
+{
+	GtkTreePath *filter_path;
+	GtkTreePath *store_path;
+	gboolean     retval = FALSE;
+
+	filter_path = gtk_tree_path_new_from_indices (pos, -1);
+	store_path = gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (gfv_list->priv->filter_model), filter_path);
+	gtk_tree_path_free (filter_path);
+
+	if (store_path == NULL)
+		return FALSE;
+
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
+				       iter,
+				       store_path))
+		retval = TRUE;
+
+	gtk_tree_path_free (store_path);
+
+	return retval;
+}
+
+
 static void
 gfv_set_image_pixbuf (GthFileView  *file_view,
 		      int           pos,
@@ -312,16 +346,9 @@ gfv_set_image_pixbuf (GthFileView  *file_view,
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
-	GtkTreePath     *path;
 
-	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
-				       &iter,
-				       path)) {
-		gtk_tree_path_free (path);
+	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
 		return;
-	}
-	gtk_tree_path_free (path);
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_ICON, get_sized_pixbuf (gfv_list, pixbuf),
@@ -335,21 +362,14 @@ gfv_set_unknown_pixbuf (GthFileView  *file_view,
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
-	GtkTreePath     *path;
 	GdkPixbuf       *real_pixbuf;
 
-	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
-				       &iter,
-				       path)) {
-		gtk_tree_path_free (path);
+	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
 		return;
-	}
-	gtk_tree_path_free (path);
 
 	if (! gfv_list->priv->enable_thumbs)
 		real_pixbuf = g_object_ref (gfv_list->priv->unknown_pixbuf_small);
-	else 
+	else
 		real_pixbuf = get_sized_pixbuf (gfv_list, gfv_list->priv->unknown_pixbuf_big);
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
@@ -367,16 +387,9 @@ gfv_set_image_text (GthFileView  *file_view,
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
-	GtkTreePath     *path;
 
-	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
-				       &iter,
-				       path)) {
-		gtk_tree_path_free (path);
+	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
 		return;
-	}
-	gtk_tree_path_free (path);
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_NAME, text,
@@ -394,7 +407,7 @@ gfv_get_image_text (GthFileView  *file_view,
 	char            *text = NULL;
 
 	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
+	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->filter_model),
 				       &iter,
 				       path)) {
 		gtk_tree_path_free (path);
@@ -402,7 +415,7 @@ gfv_get_image_text (GthFileView  *file_view,
 	}
 	gtk_tree_path_free (path);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->list_store), &iter,
+	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->filter_model), &iter,
                             COLUMN_NAME, &text,
                             -1);
 
@@ -417,16 +430,9 @@ gfv_set_image_comment (GthFileView  *file_view,
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
-	GtkTreePath     *path;
 
-	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
-				       &iter,
-				       path)) {
-		gtk_tree_path_free (path);
+	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
 		return;
-	}
-	gtk_tree_path_free (path);
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_COMMENT, comment,
@@ -444,7 +450,7 @@ gfv_get_image_comment (GthFileView  *file_view,
 	char            *comment = NULL;
 
 	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
+	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->filter_model),
 				       &iter,
 				       path)) {
 		gtk_tree_path_free (path);
@@ -452,7 +458,7 @@ gfv_get_image_comment (GthFileView  *file_view,
 	}
 	gtk_tree_path_free (path);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->list_store), &iter,
+	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->filter_model), &iter,
 			    COLUMN_COMMENT, &comment,
 			    -1);
 
@@ -464,13 +470,13 @@ static int
 gfv_get_images (GthFileView  *file_view)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
-	GtkTreeModel    *model = GTK_TREE_MODEL (gfv_list->priv->list_store);
+	GtkTreeModel    *model = GTK_TREE_MODEL (gfv_list->priv->filter_model);
 	GtkTreeIter      iter;
 	int              len = 0;
 
 	if (! gtk_tree_model_get_iter_first (model, &iter))
 		return len;
-	
+
 	do {
 		len++;
 	} while (gtk_tree_model_iter_next (model, &iter));
@@ -483,13 +489,13 @@ static GList *
 gfv_get_list (GthFileView  *file_view)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
-	GtkTreeModel    *model = GTK_TREE_MODEL (gfv_list->priv->list_store);
+	GtkTreeModel    *model = GTK_TREE_MODEL (gfv_list->priv->filter_model);
 	GtkTreeIter      iter;
 	GList           *list = NULL;
 
 	if (! gtk_tree_model_get_iter_first (model, &iter))
 		return NULL;
-	
+
 	do {
 		gpointer iter_data;
 
@@ -632,7 +638,7 @@ gfv_pos_is_selected (GthFileView     *file_view,
 	selection = gtk_tree_view_get_selection (gfv_list->priv->tree_view);
 
 	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
+	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->filter_model),
 				       &iter,
 				       path)) {
 		gtk_tree_path_free (path);
@@ -765,21 +771,14 @@ gfv_set_image_width (GthFileView     *file_view,
 
 static void
 gfv_set_image_data (GthFileView     *file_view,
-		    int              pos, 
+		    int              pos,
 		    gpointer         data)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 	GtkTreeIter      iter;
-	GtkTreePath     *path;
 
-	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
-				       &iter,
-				       path)) {
-		gtk_tree_path_free (path);
+	if (! get_list_store_iter_from_pos (gfv_list, pos, &iter))
 		return;
-	}
-	gtk_tree_path_free (path);
 
 	gtk_list_store_set (gfv_list->priv->list_store, &iter,
 			    COLUMN_FILE_DATA, data,
@@ -789,7 +788,7 @@ gfv_set_image_data (GthFileView     *file_view,
 
 static void
 gfv_set_image_data_full (GthFileView     *file_view,
-			 int              pos, 
+			 int              pos,
 			 gpointer         data,
 			 GtkDestroyNotify destroy)
 {
@@ -809,7 +808,7 @@ gfv_find_image_from_data (GthFileView     *file_view,
 
 	if (! gtk_tree_model_get_iter_first (model, &iter))
 		return -1;
-	
+
 	do {
 		gpointer iter_data;
 
@@ -836,7 +835,7 @@ gfv_get_image_data (GthFileView     *file_view,
 	FileData        *fdata;
 
 	path = gtk_tree_path_new_from_indices (pos, -1);
-	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->list_store),
+	if (! gtk_tree_model_get_iter (GTK_TREE_MODEL (gfv_list->priv->filter_model),
 				       &iter,
 				       path)) {
 		gtk_tree_path_free (path);
@@ -844,7 +843,7 @@ gfv_get_image_data (GthFileView     *file_view,
 	}
 	gtk_tree_path_free (path);
 
-	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->list_store), &iter,
+	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->filter_model), &iter,
                             COLUMN_FILE_DATA, &fdata,
                             -1);
 	file_data_ref (fdata);
@@ -894,7 +893,7 @@ gfv_get_view_mode (GthFileView *file_view)
 
 static void
 gfv_moveto (GthFileView *file_view,
-	    int          pos, 
+	    int          pos,
 	    double       yalign)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
@@ -936,20 +935,20 @@ gfv_image_is_visible (GthFileView *file_view,
 
 	else if (image_rect.y < visible_rect.y)
 		visibility = GTH_VISIBILITY_PARTIAL_TOP;
-	
+
 	else if (image_rect.y + image_rect.height > visible_rect.y + visible_rect.height)
 		visibility = GTH_VISIBILITY_PARTIAL_BOTTOM;
-	
+
 	else
 		visibility = GTH_VISIBILITY_FULL;
-	
+
 	return visibility;
 }
 
 
 static int
-gfv_get_image_at (GthFileView *file_view, 
-		  int          x, 
+gfv_get_image_at (GthFileView *file_view,
+		  int          x,
 		  int          y)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
@@ -980,11 +979,9 @@ gfv_get_first_visible (GthFileView *file_view)
 	GtkTreePath     *path;
 	int              pos;
 
-	gtk_tree_view_get_visible_rect (gfv_list->priv->tree_view,
-					&rect);
-
+	gtk_tree_view_get_visible_rect (gfv_list->priv->tree_view, &rect);
 	if (! gtk_tree_view_get_path_at_pos (gfv_list->priv->tree_view,
-					     0, 
+					     0,
 					     0,
 					     &path,
 					     NULL,
@@ -1007,10 +1004,9 @@ gfv_get_last_visible (GthFileView *file_view)
 	GtkTreePath     *path;
 	int              pos;
 
-	gtk_tree_view_get_visible_rect (gfv_list->priv->tree_view,
-					&rect);
+	gtk_tree_view_get_visible_rect (gfv_list->priv->tree_view, &rect);
 	if (! gtk_tree_view_get_path_at_pos (gfv_list->priv->tree_view,
-					     0, 
+					     0,
 					     rect.height - 1,
 					     &path,
 					     NULL,
@@ -1025,6 +1021,39 @@ gfv_get_last_visible (GthFileView *file_view)
 }
 
 
+static gboolean
+gfv_visible_func (GtkTreeModel *model,
+                  GtkTreeIter  *iter,
+                  gpointer      data)
+{
+	GthFileViewList *gfv_list = (GthFileViewList *) data;
+	FileData        *fdata;
+
+	if (gfv_list->priv->filter_func == NULL)
+		return TRUE;
+
+	gtk_tree_model_get (GTK_TREE_MODEL (gfv_list->priv->list_store), iter,
+                            COLUMN_FILE_DATA, &fdata,
+                            -1);
+
+	return (gfv_list->priv->filter_func) (gfv_list->priv->filter_data, fdata);
+}
+
+
+static void
+gfv_set_visible_func (GthFileView    *file_view,
+                      GthVisibleFunc  func,
+                      gpointer        data)
+{
+	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
+
+	gfv_list->priv->filter_func = func;
+	gfv_list->priv->filter_data = data;
+
+	gtk_tree_model_filter_refilter (GTK_TREE_MODEL_FILTER (gfv_list->priv->filter_model));
+}
+
+
 static void
 gfv_sorted (GthFileView   *file_view,
 	    GthSortMethod  sort_method,
@@ -1042,6 +1071,7 @@ gfv_sorted (GthFileView   *file_view,
 	case GTH_SORT_METHOD_BY_PATH: col = COLUMN_PATH; break;
 	case GTH_SORT_METHOD_BY_SIZE: col = COLUMN_SIZE; break;
 	case GTH_SORT_METHOD_BY_TIME: col = COLUMN_TIME; break;
+	case GTH_SORT_METHOD_BY_COMMENT: col = COLUMN_COMMENT; break;
 	default: col = GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID; break;
 	}
 
@@ -1076,7 +1106,7 @@ gfv_unsorted (GthFileView *file_view)
 
 
 static void
-gfv_image_activated (GthFileView *file_view, 
+gfv_image_activated (GthFileView *file_view,
 		     int          pos)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
@@ -1089,7 +1119,7 @@ gfv_image_activated (GthFileView *file_view,
 
 
 static void
-gfv_set_cursor (GthFileView *file_view, 
+gfv_set_cursor (GthFileView *file_view,
 		int          pos)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
@@ -1119,10 +1149,10 @@ gfv_get_cursor (GthFileView *file_view)
 static GdkPixbuf *
 create_unknown_pixbuf (GthFileViewList *gfv_list, gboolean big)
 {
-	GnomeIconTheme *icon_theme = gfv_list->priv->icon_theme;
+	GtkIconTheme   *icon_theme = gfv_list->priv->icon_theme;
+	GtkIconInfo    *icon_info = NULL;
 	int             icon_width, icon_height, icon_size;
 	char           *icon_name;
-	char           *icon_path;
 	GdkPixbuf      *pixbuf = NULL;
 	int             width, height;
 
@@ -1139,22 +1169,23 @@ create_unknown_pixbuf (GthFileViewList *gfv_list, gboolean big)
 				       "image/*",
 				       GNOME_ICON_LOOKUP_FLAGS_NONE,
 				       NULL);
-	icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-						  icon_name,
-						  icon_size,
-						  NULL, 
-						  NULL);
+
+	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
+						icon_name,
+						icon_size,
+						0);
+
 	g_free (icon_name);
 
-	if (icon_path != NULL) {
-		pixbuf = gdk_pixbuf_new_from_file (icon_path, NULL);
-		g_free (icon_path);
+	if (icon_info != NULL) {
+		pixbuf = gtk_icon_info_load_icon (icon_info, NULL);
+		gtk_icon_info_free (icon_info);
 	}
 
-	if (pixbuf == NULL) 
-		pixbuf = gdk_pixbuf_new_from_inline (-1, 
-						     dir_16_rgba, 
-						     FALSE, 
+	if (pixbuf == NULL)
+		pixbuf = gdk_pixbuf_new_from_inline (-1,
+						     dir_16_rgba,
+						     FALSE,
 						     NULL);
 
 	width = gdk_pixbuf_get_width (pixbuf);
@@ -1178,10 +1209,10 @@ gfv_update_icon_theme (GthFileView *file_view)
 {
 	GthFileViewList *gfv_list = (GthFileViewList *) file_view;
 
-	if (gfv_list->priv->unknown_pixbuf_small != NULL) 
+	if (gfv_list->priv->unknown_pixbuf_small != NULL)
 		g_object_unref (gfv_list->priv->unknown_pixbuf_small);
 
-	if (gfv_list->priv->unknown_pixbuf_big != NULL) 
+	if (gfv_list->priv->unknown_pixbuf_big != NULL)
 		g_object_unref (gfv_list->priv->unknown_pixbuf_big);
 
 	gfv_list->priv->unknown_pixbuf_small = create_unknown_pixbuf (gfv_list, FALSE);
@@ -1217,7 +1248,7 @@ gfv_set_drag_dest_pos (GthFileView *file_view,
 		pos = GTK_TREE_VIEW_DROP_AFTER;
 	else if (pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
 		pos = GTK_TREE_VIEW_DROP_BEFORE;
-	
+
 	gtk_tree_view_set_drag_dest_row (gfv_list->priv->tree_view,
 					 path,
 					 pos);
@@ -1239,7 +1270,7 @@ gfv_get_drag_dest_pos (GthFileView     *file_view,
 		*dest_pos = -1;
 		return;
 	}
-		
+
 	gtk_tree_view_get_drag_dest_row (gfv_list->priv->tree_view,
 					 &path,
 					 &pos);
@@ -1247,11 +1278,11 @@ gfv_get_drag_dest_pos (GthFileView     *file_view,
 	if (path != NULL) {
 		idx = gtk_tree_path_get_indices (path);
 		*dest_pos = idx[0];
-		if ((pos == GTK_TREE_VIEW_DROP_AFTER) 
+		if ((pos == GTK_TREE_VIEW_DROP_AFTER)
 		    || (pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER))
 			(*dest_pos)++;
 		gtk_tree_path_free (path);
-	} else 
+	} else
 		*dest_pos = -1;
 }
 
@@ -1308,7 +1339,6 @@ gth_file_view_list_finalize (GObject *object)
 
 	gfv_list = (GthFileViewList*) object;
 
-	g_object_unref (gfv_list->priv->icon_theme);
 	g_object_unref (gfv_list->priv->unknown_pixbuf_small);
 	g_object_unref (gfv_list->priv->unknown_pixbuf_big);
 
@@ -1376,6 +1406,7 @@ gth_file_view_list_class_init (GthFileViewListClass *file_view_list_class)
 	file_view_class->get_image_at         = gfv_get_image_at;
 	file_view_class->get_first_visible    = gfv_get_first_visible;
 	file_view_class->get_last_visible     = gfv_get_last_visible;
+	file_view_class->set_visible_func     = gfv_set_visible_func;
 	file_view_class->sorted               = gfv_sorted;
 	file_view_class->unsorted             = gfv_unsorted;
 	file_view_class->image_activated      = gfv_image_activated;
@@ -1433,7 +1464,7 @@ gth_file_view_list_get_type (void)
 }
 
 
-static void 
+static void
 selection_changed_cb (GtkTreeSelection *selection,
 		      GthFileViewList  *gfv_list)
 {
@@ -1475,6 +1506,7 @@ add_columns (GtkTreeView *treeview)
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
 	int                i, j;
+	GValue             value = { 0, };
 
 	/* The Name column. */
 	column = gtk_tree_view_column_new ();
@@ -1487,6 +1519,11 @@ add_columns (GtkTreeView *treeview)
                                              NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
+        g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+        g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+        g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+        g_value_unset (&value);
+
 	gtk_tree_view_column_pack_start (column,
 					 renderer,
 					 TRUE);
@@ -1494,7 +1531,7 @@ add_columns (GtkTreeView *treeview)
                                              "text", COLUMN_NAME,
                                              NULL);
 
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	/*gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);*/
 	/*
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_NAME);
 	gtk_tree_view_column_set_sort_column_id (column, COLUMN_PATH);
@@ -1531,7 +1568,7 @@ comp_func_name (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return strcasecmp (fd1->name, fd2->name);
+	return gth_sort_by_filename_but_ignore_path (fd1->name, fd2->name);
 }
 
 
@@ -1544,11 +1581,7 @@ comp_func_size (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->size < fd2->size) return -1;
-	if (fd1->size > fd2->size) return 1;
-
-	/* if the size is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+	return gth_sort_by_size_then_name (fd1->size, fd2->size, fd1->path, fd2->path);
 }
 
 
@@ -1561,11 +1594,21 @@ comp_func_time (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	if (fd1->mtime < fd2->mtime) return -1;
-	if (fd1->mtime > fd2->mtime) return 1;
+	return gth_sort_by_filetime_then_name (fd1->mtime, fd2->mtime,
+						 fd1->path, fd2->path);
+}
 
-	/* if time is the same order by name. */
-	return comp_func_name (ptr1, ptr2);
+
+static int
+comp_func_exif_date (gconstpointer  ptr1,
+		     gconstpointer  ptr2)
+{
+	FileData *fd1 = (FileData *) ptr1, *fd2 = (FileData *) ptr2;
+
+	if ((fd1 == NULL) || (fd2 == NULL))
+		return 0;
+
+	return gth_sort_by_exiftime_then_name (fd1, fd2);
 }
 
 
@@ -1578,19 +1621,21 @@ comp_func_path (gconstpointer  ptr1,
 	if ((fd1 == NULL) || (fd2 == NULL))
 		return 0;
 
-	return uricmp (fd1->path, fd2->path);
+	return gth_sort_by_full_path (fd1->path, fd2->path);
 }
 
 
 static int
-comp_func_none (gconstpointer  ptr1,
-		gconstpointer  ptr2)
+comp_func_comment (gconstpointer ptr1, gconstpointer ptr2)
 {
-	return -1;
+	const FileData *fd1 = ptr1, *fd2 = ptr2;
+
+	return gth_sort_by_comment_then_name (fd1->comment, fd2->comment,
+					      fd1->path, fd2->path);
 }
 
 
-static GCompareFunc 
+static GCompareFunc
 get_compfunc_from_method (GthSortMethod sort_method)
 {
 	GCompareFunc func;
@@ -1608,8 +1653,14 @@ get_compfunc_from_method (GthSortMethod sort_method)
 	case GTH_SORT_METHOD_BY_PATH:
 		func = comp_func_path;
 		break;
+	case GTH_SORT_METHOD_BY_COMMENT:
+		func = comp_func_comment;
+		break;
+	case GTH_SORT_METHOD_BY_EXIF_DATE:
+		func = comp_func_exif_date;
+		break;
 	default:
-		func = comp_func_none;
+		func = gth_sort_none;
 		break;
 	}
 
@@ -1618,9 +1669,9 @@ get_compfunc_from_method (GthSortMethod sort_method)
 
 
 static int
-default_sort_func (GtkTreeModel *model, 
-		   GtkTreeIter  *a, 
-		   GtkTreeIter  *b, 
+default_sort_func (GtkTreeModel *model,
+		   GtkTreeIter  *a,
+		   GtkTreeIter  *b,
 		   gpointer      user_data)
 {
 	GthFileViewList *gfv_list = user_data;
@@ -1651,7 +1702,7 @@ gth_file_view_list_new (guint image_width)
 	gfv_list = GTH_FILE_VIEW_LIST (g_object_new (GTH_TYPE_FILE_VIEW_LIST, NULL));
 	priv = gfv_list->priv;
 
-	priv->list_store = gtk_list_store_new (NUMBER_OF_COLUMNS, 
+	priv->list_store = gtk_list_store_new (NUMBER_OF_COLUMNS,
 					       G_TYPE_POINTER,
 					       GDK_TYPE_PIXBUF,
 					       G_TYPE_STRING,
@@ -1659,10 +1710,12 @@ gth_file_view_list_new (guint image_width)
 					       G_TYPE_STRING,
 					       G_TYPE_STRING,
 					       G_TYPE_STRING);
-
-	priv->tree_view = (GtkTreeView*) gtk_tree_view_new_with_model (GTK_TREE_MODEL (priv->list_store));
+	priv->filter_model =  gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->list_store), NULL);
 	g_object_unref (priv->list_store);
 
+	priv->tree_view = (GtkTreeView*) gtk_tree_view_new_with_model (priv->filter_model);
+	g_object_unref (priv->filter_model);
+
 	gtk_tree_view_set_rules_hint (priv->tree_view, FALSE);
 	gtk_tree_view_set_headers_visible (priv->tree_view, FALSE);
 
@@ -1691,6 +1744,11 @@ gth_file_view_list_new (guint image_width)
 	selection = gtk_tree_view_get_selection (priv->tree_view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
+	gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (priv->filter_model),
+						gfv_visible_func,
+						gfv_list,
+						NULL);
+
 	g_signal_connect (G_OBJECT (selection),
 			  "changed",
 			  G_CALLBACK (selection_changed_cb),
@@ -1706,8 +1764,7 @@ gth_file_view_list_new (guint image_width)
 
 	/**/
 
-	priv->icon_theme = gnome_icon_theme_new ();
-	gnome_icon_theme_set_allow_svg (priv->icon_theme, TRUE);
+	priv->icon_theme = gtk_icon_theme_get_default ();
 	priv->unknown_pixbuf_small = create_unknown_pixbuf (gfv_list, FALSE);
 	priv->unknown_pixbuf_big = create_unknown_pixbuf (gfv_list, TRUE);
 
